import { createElementWithClass } from "./util.mjs"

/**
 * Create an HTML card
 * @param {string} title The title the card should have (placed inside an <h2>)
 * @param {String | undefined} supplemental Optional: A supplemental text that is placed floating right to the title
 * @returns A div.card DOM element
 */
export function createCard(title, supplemental) {
    const cardEl = createElementWithClass("div", "card")

    const titleEl = createElementWithClass("div", "title")

    const h2El = document.createElement("h2")
    h2El.textContent = title
    titleEl.append(h2El)

    if (supplemental !== undefined) {
        const supplementalEl = createElementWithClass("span", "supplemental")
        supplementalEl.textContent = supplemental
        titleEl.append(supplementalEl)
    }

    const contentEl = createElementWithClass("div", "content")
    cardEl.append(titleEl, contentEl)

    return cardEl
}

/**
 * Set the HTML title attribute (which controls the text that appears on mouseover) of the card
 * and if the title bar should appear clickable.
 * 
 * @param {HTMLElement} cardEl The .card element
 * @param {string} text The text for the hover title
 * @param {Boolean} clickable If the title bar should appear clickable
 */
export function setCardHoverTitle(cardEl, text, clickable) {
    const titleEl = cardEl.querySelector(".title")
    titleEl.setAttribute("title", text)
    if (clickable) {
        titleEl.classList.add("clickable")
    } else {
        titleEl.classList.remove("clickable")
    }
}
