import { findParent } from "./util.mjs"

/**
 * Show (or hide) a dialog.
 * The dialog is not shown if the user has configured the dialog to not be shown again.
 * 
 * @param {string} id The id of the dialog
 * @param {boolean} shown If the dialog should be shown or hidden
 */
export function setDialogShown(id, shown) {
    const DIALOG_SHOWN = "dialog-shown"
    const dialog = document.querySelector(`#${id}`)

    if (shown && !dialog.querySelector(".dont-show-again")?.checked) {
        document.body.classList.add(DIALOG_SHOWN)
        dialog.classList.add(DIALOG_SHOWN)
    } else {
        dialog.classList.remove(DIALOG_SHOWN)
        document.body.classList.remove(DIALOG_SHOWN)
    }
}

/**
 * Initializes the "don't show this dialog again feature"
 */
export function initDontShowAgain() {
    document.querySelectorAll(".dialog .dont-show-again").forEach(el => {
        const dialog = findParent(el, ".dialog")
        const storageItemId = `${dialog.id}.dont-show-again`
        const storedStatus = localStorage.getItem(storageItemId) || false
        el.checked = storedStatus

        el.onchange = () => {
            localStorage.setItem(storageItemId, el.checked)
        }
    })
}