/** Exercises that students must pass to pass the pool test **/
import { test } from "./exercises.api.mjs"

export const practice = test("Linux-Übungsmodus", function() {
    this.welcome(`
        <h1>Willkommen zum Linux-Übungsmodus</h1>
        <p>Hier kannst du vollkommen frei mit dem Linux-Terminal auf der linken Seite herumexperimentieren. Das Terminal läuft isoliert in diesem Browsertab, <strong>du kannst also absolut nichts kaputt machen</strong> (weder auf deinem Rechner, noch bei uns) – im schlimmsten Fall lädst du die Seite neu und alles ist vergessen ;)</p>
        <p>Es gibt ein paar Aufgaben im Übungsmodus, diese sind allerdings komplett freiwillig. Falls du sie bearbeitest, solltest du sie allerdings in der angegebenen Reihenfolge lösen, da die Aufgaben aufeinander aufbauen.</p>
        <p>Zögere nicht, dich <strong>bei Fragen und Problemen jederzeit an uns zu wenden</strong> (<code>technik@cl.uni-heidelberg.de</code>) – wenn dir etwas sehr komisch vorkommt, kann es gut sein, dass das Problem bei YALIKEJAZZ liegt und nicht bei dir 😉.</p>
    `)

    this.disableHandIn()

    this.exercise("Verzeichnisinhalt anzeigen", async function () {
        this.describe(`Lass dir anzeigen, ob und wenn ja welche Dateien sich in diesem Verzeichnis befinden.`)

        await this.prepareWith("cd /root; if [[ -d tux ]]; then rm -r tux; fi")

        await this.manualConfirmation()

        this.verify().commandWas((cmd)=>cmd?.match(/ls.*/))
    })

    this.exercise("Auch versteckte Dateien anzeigen", async function () {
        this.describe(`Lass dir anzeigen, welche Dateien sich in diesem Verzeichnis befinden (inklusive versteckter Dateien).`)

        await this.prepareWith("cd /root; if [[ -d tux ]]; then rm -r tux; fi")

        await this.manualConfirmation()

        this.verify().hasOutput((outputLines) => {
            for(const line of outputLines) {
                if(line.match(/.*\.ash_history.*/)) {
                    return true
                }
            }
            
            return false
        })
    })

    this.exercise("Verzeichnis anlegen", async function () {
        this.describe(`Erstelle einen Ordner (= ein Verzeichnis) mit dem Namen "tux" (ohne Anführungszeichen).`)

        await this.prepareWith("cd /root; if [[ -d tux ]]; then rm -r tux; fi")

        await this.manualConfirmation()

        this.verify("if [[ -d tux ]]; then echo yes; else echo no; fi").hasOutput("yes")
    })

    this.exercise("Aktuelles Verzeichnis anzeigen", async function () {
        this.describe("Lass dir anzeigen, in welchem Verzeichnis du gerade arbeitest.")

        await this.prepareWith("cd /root")

        await this.manualConfirmation()

        this.verify().hasOutput("/root")
    })

    this.exercise("Verzeichnis wechseln", async function() {
        this.describe(`Wechsele in das von dir erstelle Verzeichnis "tux".`)

        await this.prepareWith("cd /root; if [[ ! -d tux ]]; then mkdir tux; fi")

        await this.manualConfirmation()

        this.verify("pwd").hasOutput("/root/tux")
    })

    this.exercise("Datei anlegen", async function () {
        this.describe(`Erstelle eine Textdatei mit dem Namen "satz" und dem Inhalt "Colorless green ideas sleep furiously"
                      (jeweils ohne Anführungszeichen).`)

        await this.prepareWith("if [[ ! -d /root/tux ]]; then mkdir /root/tux; fi; cd /root/tux; if [[ -f satz ]]; then rm satz; fi")

        await this.manualConfirmation()

        this.verify("cat satz").hasOutput("Colorless green ideas sleep furiously")
    })

    this.exercise("Inhalt ausgeben", async function () {
        this.describe(`Lasse den Inhalt der Datei "satz" in die Konsole ausgeben.`)

        await this.prepareWith("if [[ ! -d /root/tux ]]; then mkdir /root/tux; fi; cd /root/tux; echo Colorless green ideas sleep furiously > satz")

        await this.manualConfirmation()

        this.verify().terminalContains("Colorless green ideas sleep furiously")
    })

    this.exercise("Datei umbenennen", async function () {
        this.describe(`Ändere den Namen der Datei "satz" in "saetze".`)

        await this.prepareWith("if [[ ! -d /root/tux ]]; then mkdir /root/tux; fi; cd /root/tux; if [[ -f saetze ]]; then rm saetze; fi; if [[ ! -f satz ]]; then echo Colorless green ideas sleep furiously > satz; fi")

        await this.manualConfirmation()

        this.verify("if [[ -f saetze && ! -f satz ]]; then echo yes; else echo no; fi").hasOutput("yes")
    })

    this.exercise("Datei bearbeiten", async function () {
        this.describe(`Füge die Zeile "The cat is on the mat" ans Ende der Datei "saetze" an.`)

        await this.prepareWith("if [[ ! -d /root/tux ]]; then mkdir /root/tux; fi; cd /root/tux; echo Colorless green ideas sleep furiously > saetze")

        await this.manualConfirmation()

        this.verify("cat saetze").hasOutput(["Colorless green ideas sleep furiously", "The cat is on the mat"])
    })

    this.exercise("Berechtigungen anzeigen", async function () {
        this.describe(`Lass dir die aktuellen Berechtigungen für die Datei "saetze" anzeigen.`)

        await this.prepareWith("if [[ ! -d /root/tux ]]; then mkdir /root/tux; fi; cd /root/tux; echo Colorless green ideas sleep furiously > saetze; echo The cat is on the mat >> saetze; chmod 644 saetze")

        await this.manualConfirmation()

        this.verify().hasOutput((outputLines) => {
            for(const line of outputLines) {
                if(line.match(/.*-rw-r--r--.*/)) {
                    return true
                }
            }
            
            return false
        })
    })

    this.exercise("Berechtigungen anpassen", async function () {
        this.describe(`Ändere die Berechtigungen der Datei "/resources/supergeheim" so, dass der Besitzer sie lesen und schreiben kann, sonst aber niemand.`)

        await this.prepareWith(`mkdir -p /resources &&
                                echo 'Dies ist ein Geheimnis.' > /resources/supergeheim &&
                                chmod g+w /resources/supergeheim`)

        await this.manualConfirmation()

        this.verify("ls -ld /resources/supergeheim").hasOutput((outputLines) => {
            // Verify that the output of "ls -ld /resources/supergeheim" contains the correct permissions
            // (the regular expression checks that the read bits are correctly set, it doesn't care about the other's)
            return outputLines.length === 1 && outputLines[0].substring(0, 10).match(/-rw.--.--./)
        })
    })

    this.exercise("Datei löschen", async function () {
        this.describe(`Lösche die Datei "saetze".`)

        await this.prepareWith("if [[ ! -d /root/tux ]]; then mkdir /root/tux; fi; cd /root/tux; if [[ ! -f saetze ]]; then echo Colorless green ideas sleep furiously > saetze; echo The cat is on the mat >> saetze; fi; chmod 460 saetze")

        await this.manualConfirmation()

        this.verify("if [[ ! -e saetze ]]; then echo yes; else echo no; fi").hasOutput("yes")
    })

    this.exercise("Freies Spiel", async function () {
        // TODO Replace with something better
        this.describe(`Wir haben leider keine weiteren Übungsaufgaben mehr vorbereitet, aber fühle dich frei, noch weiter herumzuspielen (zum Beispiel mit Dateirechten).`)

        await this.clear()

        await this.manualConfirmation()

        this.verify().commandWas(()=>true)
    })
})

export const pooltest = test((function(){
    const now = new Date()
    const month = now.getMonth() + 1
    const year = now.getFullYear()
    const is_winterterm = month >= 10 || month < 4
    return "Pooltest "
        + (is_winterterm ? "Winter" : "Sommer")
        + "semester "
        + year.toString()
        + (is_winterterm ? ("/" + (year+1).toString()) : "")
})(), async function () {
    this.welcome(`
        <h1>Willkommen zum Pooltest!</h1>
        <p>In diesem Test sollst du nachweisen, dass du grundlegende Linux-Kenntnisse besitzt und diese auch anwenden kannst. Der Test besteht aus <strong id="exerciseCnt"></strong> Aufgaben, dauert etwa eine Stunde und wir empfehlen, ihn am Stück durchzuarbeiten.</p>
        <p>Jede Aufgabe läuft nach dem gleichen Prinzip ab: Links siehst du ein Linux-Terminal, in dem du Befehle eingeben kannst. Rechts steht eine Aufgabenstellung mit einem Ziel, das du erreichen sollst. Wie du das tust, ist dir überlassen (es gibt oft mehr als einen Weg zum Ziel). Sobald du denkst, dass du mit einer Aufgabe fertig bist, drücke auf den Knopf, um deine Lösung zu überprüfen. Falls sie richtig war, wird die Aufgabe als gelöst markiert, ansonsten kannst du es beliebig oft erneut versuchen.</p>
        </p>
        <p><strong>Du kannst nichts kaputt machen!</strong> Das Linux-Terminal läuft sicher und isoliert nur in deinem Browser und hat auf nichts Zugriff außerhalb von diesem Tab. Wenn du denkst, dass du in einer Sackgasse gelandet bist, kannst du über „Aufgabe neu beginnen“ das Terminal neustarten. Ist in einer Aufgabe von einer Datei die Rede, aber es wird kein Pfad zu dieser Datei genannt, so bleibe einfach in dem Verzeichnis, in dem du dich befindest.</p>
        
        <p>Du kannst Aufgaben auch überspringen und später wieder zu ihnen zurückkehren. Sobald du alle Aufgaben gelöst hast, kannst du durch Klick auf den stilisierten Papierflieger (<i class="bi bi-send"></i>) am linken Rand abgeben. Falls du eine Aufgabe partout nicht gelöst bekommst, kannst du auch unvollständig abgeben (sag dann am besten dazu, mit welchen Aufgaben du Probleme hattest). Wir schauen uns deinen Test dann an, und sofern er ansonsten einen guten Eindruck macht, bestehst du ihn trotzdem :)</p>

        <p>Zögere nicht, dich <strong>bei Fragen und Problemen jederzeit an uns zu wenden</strong> (<code>technik@cl.uni-heidelberg.de</code>) – ganz gleich, ob das Problem eine zu schwere oder leichte Aufgabe ist oder ein Programmierfehler in YALIKEJAZZ, wir schauen uns das an und finden gemeinsam eine Lösung.</p>
    `)

    this.exercise("Datei anlegen", async function () {
        this.describe(`Erstelle eine Datei mit dem Namen "botschaft" und dem Inhalt
        "Time flies like an arrow" (jeweils ohne
        Anführungszeichen und ohne Punkt am Ende).`)

        // Wait until the user confirms that they believe they are done with the exercise ...
        await this.manualConfirmation()

        // ... then verify the existence of the file with the correct output
        this.verify("cat botschaft").hasOutput("Time flies like an arrow")
    })

    this.exercise("Textdateien lesen und schreiben", async function () {
        this.describe(`Lasse dir den Inhalt der Datei "reverse.txt" anzeigen und schreibe den Inhalt 
        in umgekehrter Reihenfolge in eine neue Datei mit dem Namen "correct.txt"`)
    
        await this.prepareWith("echo '?was I tac a ro rac a ti saW' > reverse.txt; "+
                               // Because "rev" doesn't work in this BusyBox installation, we create a set of scripts to emulate its behavior
                               "rm /bin/rev; "+
                               "echo 'i=$(( ${#1}-1 )); while [ $i -ge 0 ]; do printf \"${1:i:1}\"; i=$(( $i-1 )); done; echo' > /tmp/revline.sh; "+
                               "echo 'while true; do read -r userInput; if [ $? != 0 ]; then break; fi; /tmp/revline.sh \"$userInput\"; done' > /tmp/revstdin.sh; "+
                               "echo 'if [ ! -z $1 ]; then cat \"$1\" | /tmp/revstdin.sh; else /tmp/revstdin.sh; fi' > /bin/rev; "+
                               "chmod +x /tmp/rev*.sh; chmod +x /bin/rev")

        await this.manualConfirmation()
    
        this.verify("cat correct.txt").hasOutput("Was it a car or a cat I saw?")
    })

    this.exercise("Ordner erstellen", async function () {
        this.describe(`Erstelle unter dem Pfad "/mnt" einen Ordner mit dem Namen "uni-heidelberg", in dem sich ein weiterer Ordner
        mit dem Namen "coli" befindet.`)

        await this.manualConfirmation()

        await this.prepareWith(`mkdir -p /val &&
                                echo 'if [ -d /mnt/uni-heidelberg/coli/ ]; then echo "yes"; else echo "no"; fi' > /val/2h7je1.sh &&
                                chmod a+x /val/2h7je1.sh`)

        this.verify('/val/2h7je1.sh').hasOutput("yes")
    })

    this.exercise("Aktuelles Arbeitsverzeichnis anzeigen", async function () {
        this.describe(`Lasse dir den absoluten Pfad ausgeben, an dem du dich gerade befindest. Verwende dazu einen Befehl, 
        auch wenn du den Pfad auf dieser Linux-Maschine bereits in der Prompt ablesen könntest.`)

        await this.prepareWith("cd /proc/tty")

        await this.manualConfirmation()

        this.verify().hasOutput("/proc/tty")
    })

    this.exercise("Kopieren vieler Dateien in ein neues Verzeichnis", async function () {
        this.describe(`Kopiere alle Dateien aus dem Verzeichnis "/resources/so_much" in einen 
        neuen Unterordner deines Home-Verzeichnisses mit dem Namen "much".`)

        await this.prepareWith(`mkdir -p /resources/so_much &&
                                cd /resources/so_much &&
                                for i in "peter" "susanne" "josef" "martin" "samira" "gerhard" "manfred" "ahmad"
                                do
                                    for j in "dateien" "dokumente" "filme" "videos" "musik" "software"
                                    do 
                                        echo "Some content" > "$i-$j"
                                    done
                                done
                                `)

        await this.manualConfirmation()

        await this.prepareWith(`mkdir -p /val &&
                                echo 'diff -r /resources/so_much /root/much &> /dev/null; if [ $? == 0 ]; then echo "yes"; else echo "no"; fi' > /val/3w8ocm.sh &&
                                chmod a+x /val/3w8ocm.sh`)

        this.verify('/val/3w8ocm.sh').hasOutput("yes")
    })

    this.exercise("Löschen und Umbenennen von Ordnern und Dateien", async function () {
        this.describe(`Gehe in den Ordner "/resources/stuff". Benenne die Datei "a_file.png" zu "b_file.png" um, 
        und lösche den Unterordner "to/" mit all seinen Inhalten.`)
    
        await this.prepareWith(`mkdir -p /resources/stuff/to/delete &&
                                touch /resources/stuff/a_file.png &&
                                cd /resources/stuff/to/delete &&
                                for i in 1 2 3 4 5 6 7 8 9 10; do touch "trashfile$i.txt"; done &&
                                cd /root`)
    
        await this.manualConfirmation()

        await this.prepareWith(`mkdir -p /val &&
                                echo 'if ( ! [ -d /resources/stuff/to ] &&
                                           [ -f /resources/stuff/b_file.png ] ); then echo "yes"; else echo "no"; fi' > /val/18ne2x.sh &&
                                chmod a+x /val/18ne2x.sh`)
    
        this.verify('/val/18ne2x.sh').hasOutput("yes")
    })

    this.exercise("Verschieben der korrekten Datei", async function () {
        this.describe(`Du bist innehalb des Verzeichnisses "Contacts" in einem Unterordner mit Dateien, die als Namen mögliche Kontaktadressen tragen.
        Verschiebe die Datei mit der korrekten Mail-Adresse der Gruppe Technik in das übergeordnete Verzeichnis. Lösche alle anderen Dateien aus
        diesem Ordner, die als Namen Mail-Adressen haben, mit denen du uns nicht erreichen kannst.`)

        await this.prepareWith(`mkdir -p /resources/Contacts/unsure && 
                                cd /resources/Contacts/unsure &&
                                touch technik@cl.uni-heidelberg.de it-support@cl.uni-heidelberg.de technik-gruppe@cl.uni-heidelberg.de`)

        await this.manualConfirmation()

        await this.prepareWith(`mkdir -p /val &&
                                echo 'if ( ! [ -f /resources/Contacts/unsure/technik@cl.uni-heidelberg.de ] &&
                                           ! [ -f /resources/Contacts/unsure/it-support@cl.uni-heidelberg.de ] &&
                                           ! [ -f /resources/Contacts/unsure/technik-gruppe@cl.uni-heidelberg.de ] &&
                                           ! [ -f /resources/Contacts/technik-gruppe@cl.uni-heidelberg.de ] &&
                                           ! [ -f /resources/Contacts/it-support@cl.uni-heidelberg.de ] &&
                                           [ -f /resources/Contacts/technik@cl.uni-heidelberg.de ] ); then echo "yes"; else echo "no"; fi' > /val/2bx821.sh &&
                                chmod a+x /val/2bx821.sh`)

        this.verify('/val/2bx821.sh').hasOutput("yes")
    })

    this.exercise("Berechtigungen einer versteckten Datei herausfinden", async function () {
        this.describe(`Im Ordner "resources" im Wurzelverzeichnis liegt eine versteckte Datei mit dem Namen ".my-little-secret".
        Finde heraus, wer auf diese Datei Lesezugriff hat.`)

        await this.prepareWith(`if [[ ! -d /resources ]]; then mkdir /resources; fi &&
                                echo 'Ich mag Pizza Hawaii' > /resources/.my-little-secret &&
                                chmod 400 /resources/.my-little-secret`)

        await this.manualConfirmation()

        this.verify().hasOutput((outputLines) => {
            for(const line of outputLines) {
                if(line.match(/.*-r--------.*\.my-little-s(ecret)?.*/)) {
                    return true
                }
            }
            
            return false
        })
    })

    this.exercise("Berechtigungen ändern", async function () {
        this.describe(`Ändere die Dateirechte von "herausforderung" so, dass der Besitzer die Datei nur lesen kann, die besitzende Gruppe die Datei lesen und schreiben kann und alle anderen gar nichts dürfen.`)

        await this.prepareWith("if [[ ! -d /root/tux ]]; then mkdir /root/tux; fi; cd /root/tux; echo Wreck a nice beach > herausforderung; echo Recognize speech >> herausforderung; chmod 644 herausforderung")

        await this.manualConfirmation()

        this.verify("ls -ld herausforderung").hasOutput((outputLines) => {
            return outputLines.length === 1 && outputLines[0].substring(0, 10).match(/-r--rw----/)
        })
    })

    this.exercise("PDF-Datei ausdrucken", async function () {
        this.describe(`Im Ordner "/home/students/robobert" befindet sich eine PDF-Datei mit dem Namen "dokument.pdf".
        Gib den Befehl an, mit dem du die Datei von einem Poolrechner aus ausdrucken würdest (keine Sorge: es wird nicht wirklich
            etwas ausgedruckt).`)

        await this.prepareWith(` if [[ ! -f /bin/lp ]]; then echo '[ "$1" != "-h" ] && echo "lp: Error - No default destination." && exit 1' > /bin/lp; echo '[ "$2" != "miller" ] && echo "lp: Error - No default destination." && exit 1' >> /bin/lp; echo '[ ! -f "$3" ] && echo "lp: Error - unable to access \\"$3\\" - No such file or directory" && exit 1' >> /bin/lp; echo 'realpath "$3" > /tmp/.file-to-print' >> /bin/lp; echo 'echo "request id is pool-printer-1337 (1 file(s))"' >> /bin/lp; chmod +x /bin/lp; fi &&
            if [[ ! -d /home/students/robobert ]]; then mkdir -p /home/students/robobert; fi &&
            touch /home/students/robobert/dokument.pdf`)

        await this.manualConfirmation()

        this.verify('cat /tmp/.file-to-print').hasOutput("/home/students/robobert/dokument.pdf")
    })

    // To add another exercise, simply copy and paste an exercise(...) block from above and adjust it.
    // Order is respected, so you can e.g. order the exercises by increasing difficulty.
})
