/** Library providing a false sense of security **/

import { State } from "./state.mjs"

/**
 * Return a hand-in token, which is a gibberish string using which the GT can 
 * verify the student successfully completed the pool test
 * @returns The hand-in token
 * @param {State} state The current state
 */
export function getHandInToken(state) {
    /*
     * Hello!
     *
     * If you are not one of the members of the Gruppe Technik
     * and reading this code, you are probably a student who was
     * asked to take the pooltest and decided to search for security
     * issues in the web application instead of just taking the test.
     * 
     * Congratulations, you've found the security issue :)
     * The token for hand-in is just the Base64 encoded current state
     * (which we will later decode to check all exercises are solved), which
     * is of course easily forged. Even if we signed this somehow, it
     * could still be forged, because everything is done client-side
     * and you have full control over the client.
     * 
     * So: What prevents you from cheating now? – Nothing.
     * 
     * But, if you have come this far inspecting our code, you probably
     * also have some experience in Linux (e.g. you know why you shouldn't
     * execute rm -rf * in your home directory). 
     * 
     * So, it doesn't really matter anyway if this app is perfectly
     * secure or not. We decided for the not-so-secure (or not-so-paranoid)
     * option, because it is easier and cleaner.
     * 
     * Looking forward to seeing you in the pool :)
     * 
     * PS: If you want, feel free to write us a message at
     * ich-habe-den-pooltest-geknackt@cl.uni-heidelberg.de
     * 
     * PPS: And if you want to see how YALIKEJAZZ 2.0 could look like (which
     * fixes this issue), feel free to have a look at
     * https://www.cl.uni-heidelberg.de/gruppetechnik/yalikejazz-docker.php
     */
    return btoa(state.toString())
}

/**
 * Generate a random session uuid, whose sole purpose it is
 * to distinguish a hand-in token from this session from a hand-in
 * token from any other session.
 */
export function generateSessionUuid() {
    return crypto.randomUUID()
}
