/** Utility functions that are not specific to YALIKEJAZZ */

/**
 * Returns if all elements of the array are equal (===)
 * @param {Array} array1 
 * @param {Array} array2 
 * @returns If the two arrays are equal or not
 */
export function arrayEquals(array1, array2) {
    if (array1.length !== array2.length) return false

    for (const i in array1) {
        if (array1[i] !== array2[i]) return false
    }

    return true
}

/**
 * Create a DOM element with the given tag and the given classes
 * @param {string} tagName The tag name of the element to create
 * @param {Array<String>} classes The class(es) the element should have
 */
export function createElementWithClass(tagName, ...classes) {
    const element = document.createElement(tagName)
    element.classList.add(...classes)
    return element
}

/**
 * Wait until "check" returns a truthy value, then, return it
 * @param {() => any} check A function that returns any value
 */
export async function waitFor(check) {
    return new Promise((resolve)=>{
        const intervalId = setInterval(checkAndResolve, 100)
    
        function checkAndResolve() {
            const predicateValue = check()
            if(predicateValue) {
                clearInterval(intervalId)
                resolve(predicateValue)
            }
        }
    })
}

/**
 * @param {HTMLElement} el An element
 * @param {string} selector A selector
 * @returns The first parent element of el matching the selector
 */
export function findParent(el, selector) {
    let parent = el.parentElement
    while(!parent?.matches(selector)) {
        parent = parent.parentElement
    }
    return parent
}
